/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */

package com.intland.codebeamer.importer.bugzilla;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.WordUtils;
import org.apache.log4j.Logger;

import com.intland.codebeamer.manager.AccessRightsException;
import com.intland.codebeamer.persistence.dto.TrackerChoiceOptionDto;
import com.intland.codebeamer.persistence.dto.TrackerDto;
import com.intland.codebeamer.persistence.dto.TrackerLayoutDto;
import com.intland.codebeamer.persistence.dto.TrackerLayoutLabelDto;
import com.intland.codebeamer.persistence.dto.UserDto;


/**
 * @author <a href="mailto:robert.enyedi@intland.com">Robert Enyedi</a>
 * @version $Id$
 */
public class Utils {

	protected static final Logger logger = Logger.getLogger(Utils.class);

	private static TrackerLayoutDto layout;

	protected static TrackerLayoutLabelDto getFieldByType(Importer importer, TrackerDto bugTracker, int id) {
		layout = importer.getApi().findTrackerLayout(importer.getToken(), bugTracker.getId());
		List/*<TrackerLayoutLabelDto>*/ fields = layout.getFields();

		for(Iterator it = fields.iterator(); it.hasNext(); ) {
			TrackerLayoutLabelDto crtField = (TrackerLayoutLabelDto) it.next();

			if(crtField.getId().intValue() == id) {
				return crtField;
			}
		}

		return null;
	}

	public static void updateLatestTrackerLayout(Importer importer) throws AccessRightsException {
		if(layout != null) {
			importer.getApi().updateTrackerLayout(importer.getToken(), layout);
			logger.debug("Tracker layout updated");
		} else {
			logger.warn("Cannot update tracker layout having NULL value");
		}
	}

	static void importTrackerLabelChoiceOptions(Importer importer, TrackerDto bugTracker, List newOptions, int labelId) throws AccessRightsException {
		importTrackerLabelChoiceOptions(importer, bugTracker, null, newOptions, labelId);
	}

	/**
	 * The status label has special handling. Its content cannot be deleted and thus only the differring imported status
	 * names should be added.
	 *
	 * @param bugTracker
	 * @param newName
	 * @param newOptions
	 * @param labelId
	 * @throws AccessRightsException
	 * @throws SQLException
	 */
	static void importTrackerLabelChoiceOptions(Importer importer, TrackerDto bugTracker, String newName, List newOptions, int labelId) throws AccessRightsException {

		TrackerLayoutLabelDto trackerField = getFieldByType(importer, bugTracker, labelId);
		TrackerChoiceOptionDto[] statusOptions = null;
		Set statusNamesSet = null;

		if(newName != null) {
			trackerField.setLabel(newName);
			updateLatestTrackerLayout(importer);
		}

		// Caching status option values
		if(labelId == TrackerLayoutLabelDto.STATUS_LABEL_ID) {
			statusOptions  = importer.getApi().findTrackerChoiceOptions(importer.getToken(), bugTracker.getId(), new Integer(TrackerLayoutLabelDto.STATUS_LABEL_ID));
			statusNamesSet = new HashSet();

			for(int i=0;i<statusOptions.length; i++) {
				statusNamesSet.add(statusOptions[i].getName().toLowerCase());
			}
		}

		List choiceOptions = new ArrayList();

		for(Iterator it = newOptions.iterator(); it.hasNext(); ) {
			String crtNewOption = (String) it.next();

			if(!isFieldValueEmpty(crtNewOption)) {
				if(statusNamesSet != null) {
					if(statusNamesSet.contains(crtNewOption.toLowerCase())) {
						// Skip existing status names
						continue;
					}

					// TODO How about making ALL label options capitalized (CB style)?

					// Capitalize as the existing status options
					crtNewOption = WordUtils.capitalize(crtNewOption.toLowerCase());
				}

				TrackerChoiceOptionDto crtChoiceOption = new TrackerChoiceOptionDto();
				crtChoiceOption.setLabelId(trackerField.getId());
				crtChoiceOption.setName(crtNewOption.trim());
				crtChoiceOption.setTracker(bugTracker);

				choiceOptions.add(crtChoiceOption);
			}
		}

		importer.getApi().setTrackerChoiceOptions(importer.getToken(), choiceOptions);
	}

	/**
	 * This method tries to locate a resolution choice option for duplicate items.
	 */
	public static TrackerChoiceOptionDto findResolutionForDuplicate(Importer importer, TrackerDto bugTracker) {
		TrackerChoiceOptionDto[] statusOptions  = importer.getApi().findTrackerChoiceOptions(importer.getToken(), bugTracker.getId(), new Integer(TrackerLayoutLabelDto.RESOLUTION_LABEL_ID));

		for(int i=0; i<statusOptions.length; i++) {
			if(statusOptions[i].getName().equalsIgnoreCase("duplicate")) {
				return statusOptions[i];
			}
		}

		// No resolution choice option found for "duplicate"
		return null;
	}

	public static boolean isFieldValueEmpty(String fieldValue) {
		if(fieldValue != null) {
			fieldValue = fieldValue.trim();

			if(fieldValue.length() > 0 && !fieldValue.equals("---")) {
				return false;
			}
		}

		return true;
	}

	public static void buildValuesMap(Importer importer, TrackerDto bugTracker, Map valuesMap, int labelId) {

		valuesMap.clear();

		TrackerChoiceOptionDto[] options = importer.getApi().findTrackerChoiceOptions(importer.getToken(), bugTracker.getId(), new Integer(labelId));

		for(int i=0;i < options.length; i++) {
			valuesMap.put(options[i].getName().toLowerCase(), options[i]);
		}
	}

	public static void splitName(UserDto crtUser) {
		// TODO This is a basic name splitting which might not work on all names

		// Split real name into first and last names
		String realName = crtUser.getFirstName();
		int spaceIdx = realName.indexOf(' ');

		if(spaceIdx != StringUtils.INDEX_NOT_FOUND) {
			// Handle case of name titles, such as Dr. Peter Dornis -> [Dr. Peter] [Dornis]
			if(spaceIdx > 0 && realName.charAt(spaceIdx - 1) == '.') {
				int tmpSpaceIdx = realName.indexOf(' ', spaceIdx + 1);

				if(tmpSpaceIdx != StringUtils.INDEX_NOT_FOUND) {
					spaceIdx = tmpSpaceIdx;
				}
			}

			crtUser.setLastName(realName.substring(spaceIdx + 1));
			crtUser.setFirstName(realName.substring(0, spaceIdx));
		}
	}

}
