/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */

package com.intland.codebeamer.importer.bugzilla.dao;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.intland.codebeamer.importer.bugzilla.Importer;
import com.intland.codebeamer.importer.bugzilla.Utils;
import com.intland.codebeamer.persistence.dto.ProjectDto;
import com.intland.codebeamer.persistence.dto.TrackerChoiceOptionDto;
import com.intland.codebeamer.persistence.dto.TrackerDto;
import com.intland.codebeamer.persistence.dto.TrackerItemDto;
import com.intland.codebeamer.persistence.dto.UserDto;


/**
 * @author <a href="mailto:robert.enyedi@intland.com">Robert Enyedi</a>
 * @version $Id$
 */
public class BugzillaDaoImpl implements BugzillaDao {

	protected static final Logger logger = Logger.getLogger(BugzillaDaoImpl.class);

	private static BugzillaDaoImpl instance;

	public static BugzillaDao getInstance() {
		if(instance == null) {
			instance = new BugzillaDaoImpl();
		}

		return instance;
	}

	public List/*<UserDto>*/ findAllUsers(Importer importer) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllUsers"), "disabled");
	}

	public List/*<ProjectDto>*/ findAllProjects(Importer importer) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllProjects"), null);
	}

	public List/*<String>*/ findAllCategories(Importer importer, ProjectDto project) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllCategories"), project.getName());
	}

	public List/*<String>*/ findAllOpSys(Importer importer, ProjectDto project) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllOpSys"), project.getName());
	}

	public List/*<String>*/ findAllPlatforms(Importer importer, ProjectDto project) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllPlatforms"), project.getName());
	}

	public List/*<String>*/ findAllResolutions(Importer importer, ProjectDto project) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllResolutions"), project.getName());
	}

	public List/*<String>*/ findAllSeverities(Importer importer, ProjectDto project) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllSeverities"), project.getName());
	}

	public List/*<String>*/ findAllStatuses(Importer importer, ProjectDto project) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllStatuses"), project.getName());
	}

	public List/*<String>*/ findAllMilestones(Importer importer, ProjectDto project) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllMilestones"), project.getName());
	}

	public List/*<String>*/ findAllVersions(Importer importer, ProjectDto project) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllVersions"), project.getName());
	}

	public List/*<String>*/ findAllKeywords(Importer importer, ProjectDto project) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllKeywords"), project.getName());
	}

	public List/*<TrackerItemDto>*/ findAllBugsData(Importer importer, TrackerDto bugTracker) throws SQLException {
		List/*<Map>*/ result = importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllBugsData"), bugTracker.getProject().getName());

		List trackerItems = new ArrayList();

		for(Iterator it = result.iterator(); it.hasNext(); ) {
			Map crtRow = (Map) it.next();

			TrackerItemDto item = new TrackerItemDto();
			trackerItems.add(item);

			item.setName((String) crtRow.get("summary"));
			item.setTracker(bugTracker);

			String priorityString = (String) crtRow.get("priority");

			// Accomodating multiple Bugzilla versions
			if(priorityString.length() == 2 && priorityString.startsWith("P")) {
				item.setPriority(new Integer(priorityString.substring(1)));
			} else {
				if(priorityString.equalsIgnoreCase("standard")) {
					item.setPriority(new Integer(3));
				} else {
					logger.warn("Unrecognized priority value: " + priorityString);
				}
			}

			// Set the severity
			String severityString = (String) crtRow.get("severity");

			if(severityString != null) {

				severityString = severityString.trim().toLowerCase();

				List severitiesList = new ArrayList();

				TrackerChoiceOptionDto severityChoiceOption = (TrackerChoiceOptionDto) importer.getSeveritiesMap().get(severityString);

				if(severityChoiceOption == null) {
					logger.warn("Cannot find severity choice option for value: " + severityString);
				} else {
					severitiesList.add(severityChoiceOption);
					item.setSeverities(severitiesList);
				}
			}

			// Set the status
			String statusString = (String) crtRow.get("status");

			if(statusString != null) {

				statusString = statusString.trim().toLowerCase();

				TrackerChoiceOptionDto statusChoiceOption = (TrackerChoiceOptionDto) importer.getStatusesMap().get(statusString);

				if(statusChoiceOption == null) {
					logger.warn("Cannot find status choice option for value: " + statusString);
				} else {
					item.setStatus(statusChoiceOption);
				}
			}

			// Set the operating system
			String opsysString = (String) crtRow.get("opsys");

			if(opsysString != null) {
				opsysString = opsysString.trim().toLowerCase();

				List opsysList = new ArrayList();

				TrackerChoiceOptionDto opsysChoiceOption = (TrackerChoiceOptionDto) importer.getOpsysMap().get(opsysString);

				if(opsysChoiceOption == null) {
					logger.warn("Cannot find opsys choice option for value: " + opsysString);
				} else {
					opsysList.add(opsysChoiceOption);
					item.setOses(opsysList);
				}
			}

			// Set the operating system
			String platformString = (String) crtRow.get("platform");

			if(platformString != null) {
				platformString = platformString.trim().toLowerCase();

				List platformList = new ArrayList();

				TrackerChoiceOptionDto platformChoiceOption = (TrackerChoiceOptionDto) importer.getPlatformsMap().get(platformString);

				if(platformChoiceOption == null) {
					logger.warn("Cannot find platform choice option for value: " + platformString);
				} else {
					platformList.add(platformChoiceOption);
					item.setPlatforms(platformList);
				}
			}

			// Set the category
			String categoryString = (String) crtRow.get("category");

			if(categoryString != null) {

				categoryString = categoryString.trim().toLowerCase();

				List categoryList = new ArrayList();

				TrackerChoiceOptionDto categoryChoiceOption = (TrackerChoiceOptionDto) importer.getCategoriesMap().get(categoryString);

				if(categoryChoiceOption == null) {
					logger.warn("Cannot find category choice option for value: " + categoryString);
				} else {
					categoryList.add(categoryChoiceOption);
					item.setCategories(categoryList);
				}
			}

			// Set the resolution
			String resolutionString = (String) crtRow.get("resolution");

			if(resolutionString != null && !Utils.isFieldValueEmpty(resolutionString)) {

				resolutionString = resolutionString.trim().toLowerCase();

				List resolutionList = new ArrayList();

				TrackerChoiceOptionDto resolutionChoiceOption = (TrackerChoiceOptionDto) importer.getResolutionsMap().get(resolutionString);

				if(resolutionChoiceOption == null) {
					logger.warn("Cannot find resolution choice option for value: " + resolutionString);
				} else {
					resolutionList.add(resolutionChoiceOption);
					item.setCategories(resolutionList);
				}
			}

			// Set the version
			String versionString = (String) crtRow.get("version");

			if(versionString != null) {

				versionString = versionString.trim().toLowerCase();

				List versionsList = new ArrayList();

				TrackerChoiceOptionDto versionChoiceOption = (TrackerChoiceOptionDto) importer.getVersionsMap().get(versionString);

				if(versionChoiceOption == null) {
					logger.warn("Cannot find version choice option for value: " + resolutionString);
				} else {
					versionsList.add(versionChoiceOption);
					item.setVersions(versionsList);
				}
			}

			String assignedToString = (String) crtRow.get("assigned_to");

			if(assignedToString != null) {

				UserDto assignedToUser = (UserDto) importer.getUsersMap().get(assignedToString.toLowerCase());

				if(assignedToUser != null) {
					List usersList = new ArrayList();
					usersList.add(assignedToUser);
					item.setAssignedTo(usersList);
				} else {
					logger.warn("Could not find assigned_to user: " + assignedToString);
				}
			}

			String reporterString = (String) crtRow.get("reported_by");

			if(reporterString != null) {

				UserDto reporterUser = (UserDto) importer.getUsersMap().get(reporterString.toLowerCase());

				if(reporterUser != null) {
					item.setSubmitter(reporterUser);
				} else {
					logger.warn("Could not find reporter user: " + reporterString);
				}
			}

			Integer bugzillaId = (Integer) crtRow.get("bugzilla_id");

			if(bugzillaId != null) {
				item.setCustomField(0,bugzillaId.toString());
			}

			Date submittedAtDate = (Date) crtRow.get("submitted_at");

			if(submittedAtDate != null) {
				item.setSubmittedAt(submittedAtDate);
			}

			Date modifiedAtDate = (Date) crtRow.get("modified_at");

			if(modifiedAtDate != null) {
				item.setModifiedAt(modifiedAtDate);
			}

			String targetMilestoneString = (String) crtRow.get("target_milestone");

			if(targetMilestoneString != null && !Utils.isFieldValueEmpty(targetMilestoneString)) {

				TrackerChoiceOptionDto targetMilestoneOption = (TrackerChoiceOptionDto) importer.getMilestonesMap().get(targetMilestoneString.toLowerCase());

				if(targetMilestoneOption != null) {
					List milestonesList = new ArrayList();
					milestonesList.add(targetMilestoneOption);
					item.setMilestones(milestonesList);
				} else {
					logger.warn("Could not find choice option for milestone: " + targetMilestoneString);
				}
			}

//			String qaContactString = (String) crtRow.get("qa_contact");
//
//			if(qaContactString != null) {
//
//				UserDto qaContactUser = (UserDto) importer.usersMap.get(qaContactString.toLowerCase());
//
//				if(qaContactUser != null) {
//
//					// TODO Is this a valid assumption?
//					item.setApprover(qaContactUser);
//				} else {
//					logger.warn("Could not find qa_contact user: " + qaContactString);
//				}
//			}

		}

		return trackerItems;
	}

	public List/*<Map>*/ findAllAttachmentsForBug(Importer importer, String bugzillaId) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllAttachmentsForBug"), bugzillaId);
	}

	public List/*<Map>*/ findAllCommentsForBug(Importer importer, String bugzillaId) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllBugComments"), bugzillaId);
	}

	public List/*<Map>*/ findAllHistoryForBug(Importer importer, String bugzillaId) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllBugHistory"), bugzillaId);
	}

	public List/*<Map>*/ findAllDependencies(Importer importer) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllDependencies"), null);
	}

	public List/*<Map>*/ findAllDuplicates(Importer importer) throws SQLException {
		return importer.getSqlMapInstance().queryForList(importer.getQueryIdProvider().getTranslatedQueryId("findAllDuplicates"), null);
	}
}
