/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */

package com.intland.codebeamer.importer.bugzilla.dao;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.log4j.Logger;

import com.ibatis.sqlmap.engine.impl.SqlMapClientImpl;
import com.intland.codebeamer.importer.bugzilla.Importer;

/**
 * @author <a href="mailto:robert.enyedi@intland.com">Robert Enyedi</a>
 * @version $Id$
 */
public class QueryIdProvider {
	private static final String OLD_BUGZILLA_QUERY_SUFFIX = "-old";

	Logger logger = Logger.getLogger(getClass());

	protected Set oldStatementIdsSet;

	private Importer importer;

	public QueryIdProvider(Importer importer) {

		this.importer = importer;
		oldStatementIdsSet = new HashSet();

		if(importer.getSqlMapInstance() instanceof SqlMapClientImpl) {

		for(Iterator it = ((SqlMapClientImpl)importer.getSqlMapInstance()).getDelegate().getMappedStatementNames();it.hasNext();) {

			String crtStatementName = (String) it.next();

			// Add only database engine specific statement names
			if(crtStatementName.endsWith(OLD_BUGZILLA_QUERY_SUFFIX)) {
				oldStatementIdsSet.add(crtStatementName);
			}
		}
		} else {
			logger.error("Could not set up dynamic query ID override.");
		}
	}

	public String getTranslatedQueryId(String queryId) {
		if(importer.isOldBugzillaVersion()) {

			String oldQueryIdString = queryId + OLD_BUGZILLA_QUERY_SUFFIX;

			if(oldStatementIdsSet.contains(oldQueryIdString)) {

				logger.debug("Using query for old Bugzilla version.");

				return oldQueryIdString;
			}
		}

		return queryId;
	}
}
